@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2006 IBM Corporation and others.
+ * Copyright (c) 2000, 2010 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -64,6 +64,9 @@ import java.io.*;
  * @see Color
  * @see ImageData
  * @see ImageLoader
+ * @see <a href="http://www.eclipse.org/swt/snippets/#image">Image snippets</a>
+ * @see <a href="http://www.eclipse.org/swt/examples.php">SWT Examples: GraphicsExample, ImageAnalyzer</a>
+ * @see <a href="http://www.eclipse.org/swt/">Sample code and further information</a>
  */
 public final class Image extends Resource implements Drawable {
 
@@ -76,6 +79,8 @@ public final class Image extends Resource implements Drawable {
 	 * within the packages provided by SWT. It is not available on all
 	 * platforms and should never be accessed from application code.
 	 * </p>
+	 * 
+	 * @noreference This field is not intended to be referenced by clients.
 	 */
 	public int type;
 	
@@ -88,6 +93,8 @@ public final class Image extends Resource implements Drawable {
 	 * within the packages provided by SWT. It is not available on all
 	 * platforms and should never be accessed from application code.
 	 * </p>
+	 * 
+	 * @noreference This field is not intended to be referenced by clients.
 	 */
 	public int /*long*/ pixmap;
 	
@@ -100,6 +107,8 @@ public final class Image extends Resource implements Drawable {
 	 * within the packages provided by SWT. It is not available on all
 	 * platforms and should never be accessed from application code.
 	 * </p>
+	 * 
+	 * @noreference This field is not intended to be referenced by clients.
 	 */
 	public int /*long*/ mask;
 
@@ -140,7 +149,8 @@ public final class Image extends Resource implements Drawable {
 	 */
 	static final int DEFAULT_SCANLINE_PAD = 4;
 
-Image() {
+Image(Device device) {
+	super(device);
 }
 
 /**
@@ -174,10 +184,9 @@ Image() {
  * </ul>
  */
 public Image(Device device, int width, int height) {
-	if (device == null) device = Device.getDevice();
-	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
-	init(device, width, height);
-	if (device.tracking) device.new_Object(this);
+	super(device);
+	init(width, height);
+	init();
 }
 
 /**
@@ -185,11 +194,11 @@ public Image(Device device, int width, int height) {
  * provided image, with an appearance that varies depending
  * on the value of the flag. The possible flag values are:
  * <dl>
- * <dt><b>IMAGE_COPY</b></dt>
+ * <dt><b>{@link SWT#IMAGE_COPY}</b></dt>
  * <dd>the result is an identical copy of srcImage</dd>
- * <dt><b>IMAGE_DISABLE</b></dt>
+ * <dt><b>{@link SWT#IMAGE_DISABLE}</b></dt>
  * <dd>the result is a copy of srcImage which has a <em>disabled</em> look</dd>
- * <dt><b>IMAGE_GRAY</b></dt>
+ * <dt><b>{@link SWT#IMAGE_GRAY}</b></dt>
  * <dd>the result is a copy of srcImage which has a <em>gray scale</em> look</dd>
  * </dl>
  *
@@ -212,8 +221,7 @@ public Image(Device device, int width, int height) {
  * </ul>
  */
 public Image(Device device, Image srcImage, int flag) {
-	if (device == null) device = Device.getDevice();
-	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+	super(device);
 	if (srcImage == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	if (srcImage.isDisposed()) SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	switch (flag) {
@@ -224,7 +232,7 @@ public Image(Device device, Image srcImage, int flag) {
 		default:
 			SWT.error(SWT.ERROR_INVALID_ARGUMENT);
 	}
-	this.device = device;
+	device = this.device;
 	this.type = srcImage.type;
 
 	/* Get source image size */
@@ -255,6 +263,7 @@ public Image(Device device, Image srcImage, int flag) {
 		alphaData = new byte[srcImage.alphaData.length];
 		System.arraycopy(srcImage.alphaData, 0, alphaData, 0, alphaData.length);
 	}
+	createAlphaMask(width, height);
 
 	/* Create the new pixmap */
 	int /*long*/ pixmap = OS.gdk_pixmap_new (OS.GDK_ROOT_PARENT(), width, height, -1);
@@ -266,85 +275,78 @@ public Image(Device device, Image srcImage, int flag) {
 	if (flag == SWT.IMAGE_COPY) {
 		OS.gdk_draw_drawable(pixmap, gdkGC, srcImage.pixmap, 0, 0, 0, 0, width, height);
 		OS.g_object_unref(gdkGC);
-		if (device.tracking) device.new_Object(this);
-		return;
-	}
+	} else {
+		
+		/* Retrieve the source pixmap data */
+		int /*long*/ pixbuf = OS.gdk_pixbuf_new(OS.GDK_COLORSPACE_RGB, false, 8, width, height);
+		if (pixbuf == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+		int /*long*/ colormap = OS.gdk_colormap_get_system();
+		OS.gdk_pixbuf_get_from_drawable(pixbuf, srcImage.pixmap, colormap, 0, 0, 0, 0, width, height);
+		int stride = OS.gdk_pixbuf_get_rowstride(pixbuf);
+		int /*long*/ pixels = OS.gdk_pixbuf_get_pixels(pixbuf);
 	
-	/* Retrieve the source pixmap data */
-	int /*long*/ pixbuf = OS.gdk_pixbuf_new(OS.GDK_COLORSPACE_RGB, false, 8, width, height);
-	if (pixbuf == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-	int /*long*/ colormap = OS.gdk_colormap_get_system();
-	OS.gdk_pixbuf_get_from_drawable(pixbuf, srcImage.pixmap, colormap, 0, 0, 0, 0, width, height);
-	int stride = OS.gdk_pixbuf_get_rowstride(pixbuf);
-	int /*long*/ pixels = OS.gdk_pixbuf_get_pixels(pixbuf);
-
-	/* Apply transformation */
-	switch (flag) {
-		case SWT.IMAGE_DISABLE: {
-			Color zeroColor = device.getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW);
-			RGB zeroRGB = zeroColor.getRGB();
-			byte zeroRed = (byte)zeroRGB.red;
-			byte zeroGreen = (byte)zeroRGB.green;
-			byte zeroBlue = (byte)zeroRGB.blue;
-			Color oneColor = device.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);
-			RGB oneRGB = oneColor.getRGB();
-			byte oneRed = (byte)oneRGB.red;
-			byte oneGreen = (byte)oneRGB.green;
-			byte oneBlue = (byte)oneRGB.blue;
-			byte[] line = new byte[stride];
-			for (int y=0; y<height; y++) {
-				OS.memmove(line, pixels + (y * stride), stride);
-				for (int x=0; x<width; x++) {
-					int offset = x*3;
-					int red = line[offset] & 0xFF;
-					int green = line[offset+1] & 0xFF;
-					int blue = line[offset+2] & 0xFF;
-					int intensity = red * red + green * green + blue * blue;
-					if (intensity < 98304) {
-						line[offset] = zeroRed;
-						line[offset+1] = zeroGreen;
-						line[offset+2] = zeroBlue;
-					} else {
-						line[offset] = oneRed;
-						line[offset+1] = oneGreen;
-						line[offset+2] = oneBlue;
+		/* Apply transformation */
+		switch (flag) {
+			case SWT.IMAGE_DISABLE: {
+				Color zeroColor = device.getSystemColor(SWT.COLOR_WIDGET_NORMAL_SHADOW);
+				RGB zeroRGB = zeroColor.getRGB();
+				byte zeroRed = (byte)zeroRGB.red;
+				byte zeroGreen = (byte)zeroRGB.green;
+				byte zeroBlue = (byte)zeroRGB.blue;
+				Color oneColor = device.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);
+				RGB oneRGB = oneColor.getRGB();
+				byte oneRed = (byte)oneRGB.red;
+				byte oneGreen = (byte)oneRGB.green;
+				byte oneBlue = (byte)oneRGB.blue;
+				byte[] line = new byte[stride];
+				for (int y=0; y<height; y++) {
+					OS.memmove(line, pixels + (y * stride), stride);
+					for (int x=0; x<width; x++) {
+						int offset = x*3;
+						int red = line[offset] & 0xFF;
+						int green = line[offset+1] & 0xFF;
+						int blue = line[offset+2] & 0xFF;
+						int intensity = red * red + green * green + blue * blue;
+						if (intensity < 98304) {
+							line[offset] = zeroRed;
+							line[offset+1] = zeroGreen;
+							line[offset+2] = zeroBlue;
+						} else {
+							line[offset] = oneRed;
+							line[offset+1] = oneGreen;
+							line[offset+2] = oneBlue;
+						}
 					}
+					OS.memmove(pixels + (y * stride), line, stride);
 				}
-				OS.memmove(pixels + (y * stride), line, stride);
+				break;
 			}
-			break;
-		}
-		case SWT.IMAGE_GRAY: {			
-			byte[] line = new byte[stride];
-			for (int y=0; y<height; y++) {
-				OS.memmove(line, pixels + (y * stride), stride);
-				for (int x=0; x<width; x++) {
-					int offset = x*3;
-					int red = line[offset] & 0xFF;
-					int green = line[offset+1] & 0xFF;
-					int blue = line[offset+2] & 0xFF;
-					byte intensity = (byte)((red+red+green+green+green+green+green+blue) >> 3);
-					line[offset] = line[offset+1] = line[offset+2] = intensity;
+			case SWT.IMAGE_GRAY: {			
+				byte[] line = new byte[stride];
+				for (int y=0; y<height; y++) {
+					OS.memmove(line, pixels + (y * stride), stride);
+					for (int x=0; x<width; x++) {
+						int offset = x*3;
+						int red = line[offset] & 0xFF;
+						int green = line[offset+1] & 0xFF;
+						int blue = line[offset+2] & 0xFF;
+						byte intensity = (byte)((red+red+green+green+green+green+green+blue) >> 3);
+						line[offset] = line[offset+1] = line[offset+2] = intensity;
+					}
+					OS.memmove(pixels + (y * stride), line, stride);
 				}
-				OS.memmove(pixels + (y * stride), line, stride);
+				break;
 			}
-			transparentPixel = srcImage.transparentPixel;
-			alpha = srcImage.alpha;
-			if (srcImage.alphaData != null) {
-				alphaData = new byte[srcImage.alphaData.length];
-				System.arraycopy(srcImage.alphaData, 0, alphaData, 0, alphaData.length);
-			}
-			break;
 		}
-	}
-
-	/* Copy data back to destination pixmap */
-	OS.gdk_pixbuf_render_to_drawable(pixbuf, pixmap, gdkGC, 0, 0, 0, 0, width, height, OS.GDK_RGB_DITHER_NORMAL, 0, 0);
 	
-	/* Free resources */
-	OS.g_object_unref(pixbuf);
-	OS.g_object_unref(gdkGC);
-	if (device.tracking) device.new_Object(this);
+		/* Copy data back to destination pixmap */
+		OS.gdk_pixbuf_render_to_drawable(pixbuf, pixmap, gdkGC, 0, 0, 0, 0, width, height, OS.GDK_RGB_DITHER_NORMAL, 0, 0);
+		
+		/* Free resources */
+		OS.g_object_unref(pixbuf);
+		OS.g_object_unref(gdkGC);
+	}
+	init();
 }
 
 /**
@@ -378,11 +380,10 @@ public Image(Device device, Image srcImage, int flag) {
  * </ul>
  */
 public Image(Device device, Rectangle bounds) {
-	if (device == null) device = Device.getDevice();
-	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+	super(device);
 	if (bounds == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
-	init(device, bounds.width, bounds.height);
-	if (device.tracking) device.new_Object(this);
+	init(bounds.width, bounds.height);
+	init();
 }
 
 /**
@@ -404,10 +405,9 @@ public Image(Device device, Rectangle bounds) {
  * </ul>
  */
 public Image(Device device, ImageData data) {
-	if (device == null) device = Device.getDevice();
-	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
-	init(device, data);
-	if (device.tracking) device.new_Object(this);
+	super(device);
+	init(data);
+	init();
 }
 
 /**
@@ -436,8 +436,7 @@ public Image(Device device, ImageData data) {
  * </ul>
  */
 public Image(Device device, ImageData source, ImageData mask) {
-	if (device == null) device = Device.getDevice();
-	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+	super(device);
 	if (source == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	if (mask == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
 	if (source.width != mask.width || source.height != mask.height) {
@@ -447,8 +446,8 @@ public Image(Device device, ImageData source, ImageData mask) {
 	ImageData image = new ImageData(source.width, source.height, source.depth, source.palette, source.scanlinePad, source.data);
 	image.maskPad = mask.scanlinePad;
 	image.maskData = mask.data;
-	init(device, image);
-	if (device.tracking) device.new_Object(this);
+	init(image);
+	init();
 }
 
 /**
@@ -500,10 +499,9 @@ public Image(Device device, ImageData source, ImageData mask) {
  * </ul>
  */
 public Image(Device device, InputStream stream) {
-	if (device == null) device = Device.getDevice();
-	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
-	init(device, new ImageData(stream));
-	if (device.tracking) device.new_Object(this);
+	super(device);
+	init(new ImageData(stream));
+	init();
 }
 
 /**
@@ -534,10 +532,14 @@ public Image(Device device, InputStream stream) {
  * </ul>
  */
 public Image(Device device, String filename) {
-	if (device == null) device = Device.getDevice();
-	if (device == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
+	super(device);
 	if (filename == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
-	this.device = device;
+	initNative(filename);
+	if (this.pixmap == 0) init(new ImageData(filename));
+	init();
+}
+
+void initNative(String filename) {
 	try {
 		int length = filename.length ();
 		char [] chars = new char [length];
@@ -545,13 +547,15 @@ public Image(Device device, String filename) {
 		byte [] buffer = Converter.wcsToMbcs(null, chars, true);
 		int /*long*/ pixbuf = OS.gdk_pixbuf_new_from_file(buffer, null);
 		if (pixbuf != 0) {
-			int /*long*/ [] pixmap_return = new int /*long*/ [1];
-			OS.gdk_pixbuf_render_pixmap_and_mask(pixbuf, pixmap_return, null, 0);
-			this.type = SWT.BITMAP;
-			this.pixmap = pixmap_return[0];
-			if (pixmap == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 			boolean hasAlpha = OS.gdk_pixbuf_get_has_alpha(pixbuf);
 			if (hasAlpha) {
+				/*
+				* Bug in GTK. Depending on the image (seems to affect images that have
+				* some degree of transparency all over the image), gdk_pixbuff_render_pixmap_and_mask()
+				* will return a corrupt pixmap. To avoid this, read in and store the alpha channel data
+				* for the image and then set it to 0xFF to prevent any possible corruption from 
+				* gdk_pixbuff_render_pixmap_and_mask(). 
+				*/
 				int width = OS.gdk_pixbuf_get_width(pixbuf);
 				int height = OS.gdk_pixbuf_get_height(pixbuf);
 				int stride = OS.gdk_pixbuf_get_rowstride(pixbuf);
@@ -562,36 +566,51 @@ public Image(Device device, String filename) {
 					OS.memmove(line, pixels + (y * stride), stride);
 					for (int x = 0; x < width; x++) {
 						alphaData[y*width+x] = line[x*4 + 3];
+						line[x*4 + 3] = (byte) 0xFF;
 					}
+					OS.memmove(pixels + (y * stride), line, stride);
 				}
-				if (device.useXRender) {
-					mask = OS.gdk_pixmap_new(0, width, height, 8);
-					if (mask == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-					GdkImage gdkImage = new GdkImage();
-					int /*long*/ imagePtr = OS.gdk_drawable_get_image(mask, 0, 0, width, height);
-					OS.memmove(gdkImage, imagePtr);
-					if (gdkImage.bpl == width) {
-						OS.memmove(gdkImage.mem, alphaData, alphaData.length);
-					} else {
-						line = new byte[gdkImage.bpl];
-						for (int y = 0; y < height; y++) {
-							System.arraycopy(alphaData, width * y, line, 0, width);
-							OS.memmove(gdkImage.mem + (gdkImage.bpl * y), line, gdkImage.bpl);
-						}
-					}
-					int /*long*/ gc = OS.gdk_gc_new(mask);
-					if (gc == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-					OS.gdk_draw_image(mask, gc, imagePtr, 0, 0, 0, 0, width, height);
-					OS.g_object_unref(imagePtr);
-					OS.g_object_unref(gc);
-				}
+				createAlphaMask(width, height);
 			}
+			int /*long*/ [] pixmap_return = new int /*long*/ [1];
+			OS.gdk_pixbuf_render_pixmap_and_mask(pixbuf, pixmap_return, null, 0);
+			this.type = SWT.BITMAP;
+			this.pixmap = pixmap_return[0];
+			if (pixmap == 0) SWT.error(SWT.ERROR_NO_HANDLES);
 			OS.g_object_unref (pixbuf);
-			return;
 		}
 	} catch (SWTException e) {}
-	init(device, new ImageData(filename));
-	if (device.tracking) device.new_Object(this);
+}
+
+void createAlphaMask (int width, int height) {
+	if (device.useXRender && (alpha != -1 || alphaData != null)) {
+		mask = OS.gdk_pixmap_new(0, alpha != -1 ? 1 : width, alpha != -1 ? 1 : height, 8);
+		if (mask == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+		int /*long*/ gc = OS.gdk_gc_new(mask);
+		if (alpha != -1) {
+			GdkColor color = new GdkColor();
+			color.pixel = (alpha & 0xFF) << 8 | (alpha & 0xFF);
+			OS.gdk_gc_set_foreground(gc, color);
+			OS.gdk_draw_rectangle(mask, gc, 1, 0, 0, 1, 1);
+		} else {
+			int /*long*/ imagePtr = OS.gdk_drawable_get_image(mask, 0, 0, width, height);
+			if (imagePtr == 0) SWT.error(SWT.ERROR_NO_HANDLES);
+			GdkImage gdkImage = new GdkImage();
+			OS.memmove(gdkImage, imagePtr);
+			if (gdkImage.bpl == width) {
+				OS.memmove(gdkImage.mem, alphaData, alphaData.length);
+			} else {
+				byte[] line = new byte[gdkImage.bpl];
+				for (int y = 0; y < height; y++) {
+					System.arraycopy(alphaData, width * y, line, 0, width);
+					OS.memmove(gdkImage.mem + (gdkImage.bpl * y), line, gdkImage.bpl);
+				}
+			}
+			OS.gdk_draw_image(mask, gc, imagePtr, 0, 0, 0, 0, width, height);
+			OS.g_object_unref(imagePtr);
+		}		
+		OS.g_object_unref(gc);
+	}
 }
 
 /**
@@ -632,6 +651,17 @@ void createSurface() {
 		int stride = OS.gdk_pixbuf_get_rowstride(pixbuf);
 		int /*long*/ pixels = OS.gdk_pixbuf_get_pixels(pixbuf);		
 		byte[] line = new byte[stride];
+		int /*long*/ ptr = OS.malloc(4);
+		OS.memmove(ptr, new int[]{1}, 4);
+		OS.memmove(line, ptr, 1);
+		OS.free(ptr);
+		int oa, or, og, ob;
+		boolean bigendian = line[0] == 0;
+		if (bigendian) {
+			oa = 0; or = 1; og = 2; ob = 3;
+		} else {
+			oa = 3; or = 2; og = 1; ob = 0;
+		}
 		if (mask != 0 && OS.gdk_drawable_get_depth(mask) == 1) {
 			int /*long*/ maskPixbuf = OS.gdk_pixbuf_new(OS.GDK_COLORSPACE_RGB, false, 8, width, height);
 			if (maskPixbuf == 0) SWT.error(SWT.ERROR_NO_HANDLES);
@@ -646,10 +676,15 @@ void createSurface() {
 				for (int x=0, offset1=0; x<width; x++, offset1 += 4) {
 					if (maskLine[x * 3] == 0) {
 						line[offset1 + 0] = line[offset1 + 1] = line[offset1 + 2] = line[offset1 + 3] = 0;
+					} else {
+						byte r = line[offset1 + 0];
+						byte g = line[offset1 + 1];
+						byte b = line[offset1 + 2];
+						line[offset1 + oa] = (byte)0xFF;
+						line[offset1 + or] = r;
+						line[offset1 + og] = g;
+						line[offset1 + ob] = b;
 					}
-					byte temp = line[offset1];
-					line[offset1] = line[offset1 + 2];
-					line[offset1 + 2] = temp;
 				}
 				OS.memmove(offset, line, stride);
 				offset += stride;
@@ -661,7 +696,6 @@ void createSurface() {
 			for (int y=0; y<height; y++) {
 				OS.memmove(line, offset, stride);
 				for (int x=0, offset1=0; x<width; x++, offset1 += 4) {
-					line[offset1+3] = (byte)alpha;
 					/* pre-multiplied alpha */
 					int r = ((line[offset1 + 0] & 0xFF) * alpha) + 128;
 					r = (r + (r >> 8)) >> 8;
@@ -669,9 +703,10 @@ void createSurface() {
 					g = (g + (g >> 8)) >> 8;
 					int b = ((line[offset1 + 2] & 0xFF) * alpha) + 128;
 					b = (b + (b >> 8)) >> 8;
-					line[offset1 + 0] = (byte)b;
-					line[offset1 + 1] = (byte)g;
-					line[offset1 + 2] = (byte)r;
+					line[offset1 + oa] = (byte)alpha;
+					line[offset1 + or] = (byte)r;
+					line[offset1 + og] = (byte)g;
+					line[offset1 + ob] = (byte)b;
 				}
 				OS.memmove(offset, line, stride);
 				offset += stride;
@@ -682,7 +717,6 @@ void createSurface() {
 				OS.memmove (line, offset, stride);
 				for (int x=0, offset1=0; x<width; x++, offset1 += 4) {
 					int alpha = alphaData [y*w [0]+x] & 0xFF;
-					line[offset1+3] = (byte)alpha;
 					/* pre-multiplied alpha */
 					int r = ((line[offset1 + 0] & 0xFF) * alpha) + 128;
 					r = (r + (r >> 8)) >> 8;
@@ -690,9 +724,10 @@ void createSurface() {
 					g = (g + (g >> 8)) >> 8;
 					int b = ((line[offset1 + 2] & 0xFF) * alpha) + 128;
 					b = (b + (b >> 8)) >> 8;
-					line[offset1 + 0] = (byte)b;
-					line[offset1 + 1] = (byte)g;
-					line[offset1 + 2] = (byte)r;
+					line[offset1 + oa] = (byte)alpha;
+					line[offset1 + or] = (byte)r;
+					line[offset1 + og] = (byte)g;
+					line[offset1 + ob] = (byte)b;
 				}
 				OS.memmove (offset, line, stride);
 				offset += stride;
@@ -702,10 +737,13 @@ void createSurface() {
 			for (int y = 0; y < h [0]; y++) {
 				OS.memmove (line, offset, stride);
 				for (int x=0, offset1=0; x<width; x++, offset1 += 4) {
-					line[offset1+3] = (byte)0xFF;
-					byte temp = line[offset1];
-					line[offset1] = line[offset1 + 2];
-					line[offset1 + 2] = temp;
+					byte r = line[offset1 + 0];
+					byte g = line[offset1 + 1];
+					byte b = line[offset1 + 2];
+					line[offset1 + oa] = (byte)0xFF;
+					line[offset1 + or] = r;
+					line[offset1 + og] = g;
+					line[offset1 + ob] = b;
 				}
 				OS.memmove (offset, line, stride);
 				offset += stride;
@@ -734,14 +772,7 @@ void destroyMask() {
 	mask = 0;
 }
 
-/**
- * Disposes of the operating system resources associated with
- * the image. Applications must dispose of all images which
- * they allocate.
- */
-public void dispose () {
-	if (pixmap == 0) return;
-	if (device.isDisposed()) return;
+void destroy() {
 	if (memGC != null) memGC.dispose();
 	if (pixmap != 0) OS.g_object_unref(pixmap);
 	if (mask != 0) OS.g_object_unref(mask);
@@ -749,8 +780,6 @@ public void dispose () {
 	if (surfaceData != 0) OS.g_free(surfaceData);
 	surfaceData = surface = pixmap = mask = 0;
 	memGC = null;
-	if (device.tracking) device.dispose_Object(this);
-	device = null;
 }
 
 /**
@@ -848,8 +877,7 @@ public ImageData getImageData() {
 	OS.g_object_unref(pixbuf);
 
 	PaletteData palette = new PaletteData(0xFF0000, 0xFF00, 0xFF);
-	ImageData data = new ImageData(width, height, 24, palette);
-	data.data = srcData;
+	ImageData data = new ImageData(width, height, 24, palette, 4, srcData);
 	data.bytesPerLine = stride;
 
 	if (transparentPixel == -1 && type == SWT.ICON && mask != 0) {
@@ -904,15 +932,13 @@ public ImageData getImageData() {
  * @param pixmap the OS handle for the image
  * @param mask the OS handle for the image mask
  *
- * @private
+ * @noreference This method is not intended to be referenced by clients.
  */
 public static Image gtk_new(Device device, int type, int /*long*/ pixmap, int /*long*/ mask) {
-	if (device == null) device = Device.getDevice();
-	Image image = new Image();
+	Image image = new Image(device);
 	image.type = type;
 	image.pixmap = pixmap;
 	image.mask = mask;
-	image.device = device;
 	return image;
 }
 
@@ -930,11 +956,10 @@ public int hashCode () {
 	return (int)/*64*/pixmap;
 }
 
-void init(Device device, int width, int height) {
+void init(int width, int height) {
 	if (width <= 0 || height <= 0) {
 		SWT.error (SWT.ERROR_INVALID_ARGUMENT);
 	}
-	this.device = device;
 	this.type = SWT.BITMAP;
 
 	/* Create the pixmap */
@@ -954,9 +979,8 @@ void init(Device device, int width, int height) {
 	OS.gdk_colormap_free_colors(colormap, white, 1);
 }
 
-void init(Device device, ImageData image) {
+void init(ImageData image) {
 	if (image == null) SWT.error(SWT.ERROR_NULL_ARGUMENT);
-	this.device = device;
 	int width = image.width;
 	int height = image.height;
 	PaletteData palette = image.palette;
@@ -1036,33 +1060,7 @@ void init(Device device, ImageData image) {
 			this.alphaData = new byte[image.alphaData.length];
 			System.arraycopy(image.alphaData, 0, this.alphaData, 0, alphaData.length);
 		}
-		if (device.useXRender && (alpha != -1 || alphaData != null)) {
-			mask = OS.gdk_pixmap_new(0, alpha != -1 ? 1 : width, alpha != -1 ? 1 : height, 8);
-			if (mask == 0) SWT.error(SWT.ERROR_NO_HANDLES);
-			int /*long*/ gc = OS.gdk_gc_new(mask);
-			if (alpha != -1) {
-				GdkColor color = new GdkColor();
-				color.pixel = (alpha & 0xFF) << 8 | (alpha & 0xFF);
-				OS.gdk_gc_set_foreground(gc, color);
-				OS.gdk_draw_rectangle(mask, gc, 1, 0, 0, 1, 1);
-			} else {
-				int /*long*/ imagePtr = OS.gdk_drawable_get_image(mask, 0, 0, width, height);
-				GdkImage gdkImage = new GdkImage();
-				OS.memmove(gdkImage, imagePtr);
-				if (gdkImage.bpl == width) {
-					OS.memmove(gdkImage.mem, alphaData, alphaData.length);
-				} else {
-					byte[] line = new byte[gdkImage.bpl];
-					for (int y = 0; y < height; y++) {
-						System.arraycopy(alphaData, width * y, line, 0, width);
-						OS.memmove(gdkImage.mem + (gdkImage.bpl * y), line, gdkImage.bpl);
-					}
-				}
-				OS.gdk_draw_image(mask, gc, imagePtr, 0, 0, 0, 0, width, height);
-				OS.g_object_unref(imagePtr);
-			}		
-			OS.g_object_unref(gc);
-		}
+		createAlphaMask(width, height);
 	}
 	this.pixmap = pixmap;
 }
@@ -1079,6 +1077,8 @@ void init(Device device, ImageData image) {
  *
  * @param data the platform specific GC data 
  * @return the platform specific GC handle
+ * 
+ * @noreference This method is not intended to be referenced by clients.
  */
 public int /*long*/ internal_new_GC (GCData data) {
 	if (pixmap == 0) SWT.error(SWT.ERROR_GRAPHIC_DISPOSED);
@@ -1090,12 +1090,16 @@ public int /*long*/ internal_new_GC (GCData data) {
 		int mask = SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT;
 		if ((data.style & mask) == 0) {
 			data.style |= SWT.LEFT_TO_RIGHT;
+		} else {
+			if ((data.style & SWT.RIGHT_TO_LEFT) != 0) {
+				data.style |= SWT.MIRRORED;
+			}
 		}
 		data.device = device;
 		data.drawable = pixmap;
 		data.background = device.COLOR_WHITE.handle;
 		data.foreground = device.COLOR_BLACK.handle;
-		data.font = device.systemFont.handle;
+		data.font = device.systemFont;
 		data.image = this;
 	}
 	return gdkGC;
@@ -1112,7 +1116,9 @@ public int /*long*/ internal_new_GC (GCData data) {
  * </p>
  *
  * @param hDC the platform specific GC handle
- * @param data the platform specific GC data 
+ * @param data the platform specific GC data
+ * 
+ * @noreference This method is not intended to be referenced by clients.
  */
 public void internal_dispose_GC (int /*long*/ gdkGC, GCData data) {
 	OS.g_object_unref(gdkGC);
@@ -1124,7 +1130,7 @@ public void internal_dispose_GC (int /*long*/ gdkGC, GCData data) {
  * <p>
  * This method gets the dispose state for the image.
  * When an image has been disposed, it is an error to
- * invoke any other method using the image.
+ * invoke any other method (except {@link #dispose()}) using the image.
  *
  * @return <code>true</code> when the image is disposed and <code>false</code> otherwise
  */
